{#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#}

{% extends "page.html" %}

{#
{% set title = title %}
#}
{% block title %}
{% endblock title %}

{% block metaheader %}
  {% if recid != -1 and config.CFG_WEBSEARCH_DETAILED_META_FORMAT %}
  {# format_record(record, of=config.CFG_WEBSEARCH_DETAILED_META_FORMAT, ln = g.ln)|safe #}
  {% endif %}
{% endblock metaheader %}

{%- block header %}
  {%- include "base/scripts.html" %}
  {{- super() }}
{%- endblock header %}
{%- block _bottom_assets %}
  {# The legacy page requires that the scripts are loaded from the <head>
   # rather than at the bottom of the page.
   #}
{%- endblock %}

{% block global_bundles %}
  {{ super() }}
  {% bundles "records.js", "formatter.css", "record.css" %}
{% endblock %}

{% block body %}
  {{ super() }}

  {% block record_restriction_flag %}
  {% if g.bibrec.is_restricted %}
  <div class="alert alert-danger"><b>{{ _('Restricted') }}</b>
    {%- if g.bibrec.is_processed %} {{ _('Processed Record') }}
    {%- endif -%}</div>
  {% endif %}
  {% endblock %}

  {% block record_tabs %}
  <ul id="record-links" class="nav nav-tabs">
  {%- for tab in current_menu.submenu('record').children -%}
  {%- if tab.visible -%}
    <li class="{{ 'active' if tab.active else '' }}">
      <a href="{{ tab.url }}">
        {{ tab.text }}
        {% if tab.count %}
          {% set tab_count = tab.count() %}
          {% if tab_count > -1 %}
            &nbsp;<small class="text-muted">({{ tab_count }})</small>
          {% endif %}
        {% endif %}
      </a>
    </li>
  {%- endif -%}
  {%- endfor -%}
  </ul>
  {% endblock %}

  <div id="record_content">
  {% block record_content %}
    <!-- record content -->
  {% endblock %}
  </div>
<div style="clear: both;"></div>
{% endblock %}
